import os
import re
import readline
import click
from pyocs import pyocs_jenkins
from pyocs import pyocs_software
from pyocs import __version__
from pyocs.pyocs_demand import PyocsDemand
from pyocs.pyocs_auto_config import PyocsAutoConfig
from pyocs.pyocs_filesystem import PyocsFileSystem
import pprint
from pyocs.pyocs_exception import *


STYLE = {
    'fore':
    {   # 前景色
     'black'    : 30,   #  黑色
     'red'      : 31,   #  红色
     'green'    : 32,   #  绿色
     'yellow'   : 33,   #  黄色
     'blue'     : 34,   #  蓝色
     'purple'   : 35,   #  紫红色
     'cyan'     : 36,   #  青蓝色
     'white'    : 37,   #  白色
     },

    'back' :
    {   # 背景
     'black'     : 40,  #  黑色
     'red'       : 41,  #  红色
     'green'     : 42,  #  绿色
     'yellow'    : 43,  #  黄色
     'blue'      : 44,  #  蓝色
     'purple'    : 45,  #  紫红色
     'cyan'      : 46,  #  青蓝色
     'white'     : 47,  #  白色
     },

    'mode' :
    {   # 显示模式
     'mormal'    : 0,   #  终端默认设置
     'bold'      : 1,   #  高亮显示
     'underline' : 4,   #  使用下划线
     'blink'     : 5,   #  闪烁
     'invert'    : 7,   #  反白显示
     'hide'      : 8,   #  不可见
     },

    'default' :
    {
        'end' : 0,
    },
}


def use_style(string, mode='', fore='', back=''):
    mode = '%s' % STYLE['mode'][mode] if mode in STYLE['mode'] else ''
    fore = '%s' % STYLE['fore'][fore] if fore in STYLE['fore'] else ''
    back = '%s' % STYLE['back'][back] if back in STYLE['back'] else ''
    style = ';'.join([s for s in [mode, fore, back] if s])
    style = '\033[%sm' % style if style else ''
    end = '\033[%sm' % STYLE['default']['end'] if style else ''
    return '%s%s%s' % (style, string, end)


# 使用样式输出
# print(use_style("hello world!", mode="bold"))
# print(use_style("hello world!", back="green"))


def print_version(ctx, param, value):
    if not value or ctx.resilient_parsing:
        return
    click.echo(__version__.version)
    ctx.exit()


@click.group()
@click.option('--version', is_flag=True, callback=print_version,
              expose_value=False, is_eager=True, help="打印Pyocs的版本信息")
def cli():
    """ Pyocs命令行工具，使用介绍：https://kb.cvte.com/pages/viewpage.action?pageId=135433093"""
    pass


@click.command()
def update():
    """更新Pyocs版本"""
    os.system("pip3 install --upgrade pyocs --user")


@click.command()
@click.option('--ocs', help='OCS订单号')
@click.option('--test', default='F', help='测试类型N、A、B、C、D、E、F')
@click.option('--message', default='Jenkins Auto Config System', help='OCS备注')
@click.option('--bin', default='False', help='是否做bin(True/False)')
def autoconf(ocs, test, message, bin):
    """获取OCS订单需求,生成配置，自动提交,返回model id"""
    try:
        obj = PyocsAutoConfig()
        para = obj.auto_config_by_ocs(ocs_number=ocs, test=test, message=message, is_make_bin=bin)
        click.echo(para)
    except ProjectNoSupportError:
        click.echo("Fail:暂不支持此方案，请勿重复提交")
    except CustomerNoSupportError:
        click.echo("Fail:暂不支持此客户，请勿重复提交")
    except NoRegionError:
        click.echo("Fail:订单无区域信息，请确认推动解决区域问题")
    except OcsRegionError:
        click.echo("Fail:订单区域错误，请推动解决区域问题")
    except ReadPwmError:
        click.echo("Fail:占空比错误，请检查订单占空比格式是否正确")
    except No358GaiaAI:
        click.echo("Fail:358 没有导入GAIA AI，请确认")
    except RuntimeError:
        raise


@click.command()
@click.option('--ocs', help='OCS订单号')
@click.option('--excel', help='excel参数')
@click.option('--workspace', help='工作路径')
@click.option('--test', default='F', help='测试类型N、A、B、C、D、E、F')
@click.option('--message', default='Jenkins Auto Config System', help='OCS备注')
@click.option('--bin', default='False', help='是否做bin(True/False)')
@click.option('--check', default='False', help='预检资源')
def autodt(ocs, excel, workspace, test, message, bin, check):
    """获取OCS订单需求,生成配置，自动提交,返回model id"""
    obj = PyocsAutoConfig()
    if check == 'False':
        para = obj.auto_config_by_excel(ocs_num=ocs, excel_para=excel, workspace=workspace, test=test,
                                        message=message, is_make_bin=bin)
        click.echo(para)
    else:
        ret = obj.pre_check_for_customer_demand_excel_resource(ocs_num=ocs, excel_para=excel, workspace=workspace)
        click.echo(ret)


@click.command()
@click.option('--ocs', help='OCS订单号')
@click.option('--content', default='demand', help='需要从订单上获取内容类型：例如：title(订单摘要)、demand(订单需求)，默认为demand')
def require(ocs, content):
    """获取OCS订单的简化信息"""
    if content == 'title':
        title = PyocsDemand(ocs).get_ocs_title()
        click.echo(title)
    elif content == 'demand':
        ret = PyocsDemand(ocs).get_request()
        pprint.PrettyPrinter(indent=4).pprint(ret)
    elif content == 'project':
        project = PyocsDemand(ocs).get_ocs_project_name()
        click.echo(project)
    elif content == 'customer_batch_code':
        customer_batch_code = PyocsDemand(ocs).get_customer_batch_code()
        click.echo(customer_batch_code)
    elif content == 'flash_description':
        flash_descriptrion = PyocsDemand(ocs).get_flash_model()
        click.echo(flash_descriptrion)


@click.command()
@click.argument('model')
def jenkins(model):
    """向Jenkins FAE软件云编译提交编译请求"""
    j = pyocs_jenkins.PyocsJenkins()
    j.create_jenkins_job(model)


@click.command()
@click.option('--sw', type=str, help='需要获取下载链接的软件全名')
def download(sw):
    """获取软件外网下载链接"""
    _search_task = pyocs_software.PyocsSoftware()
    result = _search_task.get_download_link_by_software_name(sw)
    print("软件包名：" + result.name)
    print("下载链接：" + result.download_link + " 有效截止时间：" + result.deadline)


@click.command()
@click.option('--sw', type=str, help='需要刷新的版本')
@click.option('--txt', type=str, help='批量刷新的txt文件')
def refresh(txt, sw):
    """刷新软件下载链接"""
    refresh_task = pyocs_software.PyocsSoftware()
    if txt and ".txt" in txt:
        """从文件批量刷新下载链接"""
        sw_list = []
        with open(txt) as file:
            for index in file.readlines():
                if index is not None:
                    sw_list.append(index.strip('\n'))
        for sw_info in sw_list:
            result = refresh_task.get_refresh_software_download_link_by_sw_info(sw_info)
            if not result:
                print(sw_info + "软件未找到，请确认软件版本是否正确!")
            else:
                for sw_download in result:
                    print("刷新软件：" + sw_info)
                    print("软件包名：" + sw_download.name)
                    print("下载链接：" + sw_download.download_link + " 有效截止时间：" + sw_download.deadline + "\n")
    elif sw:
        """只刷新一个软件版本"""
        result = refresh_task.get_refresh_software_download_link_by_sw_info(sw)
        if not result:
            print("软件未找到，请确认软件版本是否正确!")
        else:
            for sw_download in result:
                print("软件包名：" + sw_download.name)
                print("下载链接：" + sw_download.download_link + " 有效截止时间：" + sw_download.deadline)


@click.command()
@click.option('--ocs', help='需要做bin的订单')
@click.option('--sw', help='需要做bin的软件名')
def autobin(ocs, sw):
    """ --ocs 传入订单号，对此订单上的启用的软件发起做bin
    或  --sw 传入软件名，对此软件发起做bin
    """
    if ocs:
        sw_module = pyocs_software.PyocsSoftware()
        sw_list = sw_module.get_enable_software_list(ocs)
        if sw_list:
            for sw in sw_list:
                while 1:
                    ret = input("请确认做bin软件名：" + sw.name + "是否正确（Y/N）：")
                    if ret is 'Y' or ret is 'y':
                        sw_module.set_master_sw(sw.fw_id)
                        ret = sw_module.auto_make_bin(ocs, sw.attachment_id)
                        if ret:
                            print("成功发起做bin")
                        else:
                            print("做bin失败，请重新尝试")
                        break
                    elif ret is 'N' or ret is 'n':
                        break
                    else:
                        print('输入错误，请重新输入！！！')
                        continue
    elif sw:
        sw_module = pyocs_software.PyocsSoftware()
        ret = sw_module.auto_make_bin_by_sw_name(sw)
        if ret:
            print("success")
        else:
            print("failed")


@click.command()
@click.option('--content', help='需要获取的资源,常给jenkins用')
def resource(content):
    """获取一些资源"""
    if content == 'signature':
        ret = pyocs_jenkins.PyocsJenkins.get_jenkins_mail_signature()
        click.echo(ret)


@click.command()
@click.option('--ocs', help='ocs订单号')
@click.option('--type', default='order', help='备注类型，order:订单备注；fac:工厂测试备注')
@click.argument("message")
def comment(ocs, type, message):
    """给订单增加备注信息，eg: pyocs comment --ocs=434390 message"""
    sw = pyocs_software.PyocsSoftware()
    if ocs is None:
        click.echo("参照 --ocs=434390 传入需要增加备注信息的ocs订单号")
    elif type == 'fac':
        sw.comment_factory_test_info(ocs_num=ocs, message=message)
    elif type == 'order':
        sw.add_comment_to_ocs(ocs_number=ocs, message=message)
    else:
        click.echo("还未添加此种类型的备注功能")


@click.command()
@click.option('--software', help='软件所在的路径（相对路径或者绝对路径）')
@click.option('--old', help='库存软件名（软件名或版本号）')
@click.argument("ocs")
def upload(ocs, software, old):
    """给订单上传软件，库存软件支持批量上传"""
    if old:  # 如果传参old则上传库存软件
        ocs_list = ocs.split(",")
        sw = pyocs_software.PyocsSoftware()
        old_sw_dict = sw.find_old_sw_id_by_name(old_sw_name=old)
        if old_sw_dict:
            for old_sw_id, old_sw_name in old_sw_dict.items():
                while 1:
                    ret = input("请确认库存软件名：" + old_sw_name + "是否正确（Y/N）：")
                    if ret not in {'Y', 'y', 'N', 'n'}:
                        continue
                    else:
                        break
                if ret is 'Y' or ret is 'y':
                    for ocs_number in ocs_list:
                        is_success = sw.upload_old_sw_by_id(ocs_num=ocs_number, old_sw_id=old_sw_id)
                        sta = ocs_number
                        sta += "上传成功" if is_success else "上传失败"
                        click.echo(sta)
                    break
                elif ret is 'N' or ret is 'n':
                    continue
        else:
            click.echo("无法搜索到库存软件，请自行检查原因")
            os.sys.exit(0)
        os.sys.exit(1)

    if software is None:
        code_root_path = PyocsFileSystem.get_current_code_root_dir()
        if code_root_path:
            ab_code_root_path = os.path.abspath(code_root_path)
            bin_sw_path = ab_code_root_path + '/bin'
            images_sw_path = ab_code_root_path + '/images'
            if os.path.exists(bin_sw_path):
                software = bin_sw_path
            elif os.path.exists(images_sw_path):
                software = images_sw_path
            for files in os.listdir(software):
                if os.path.splitext(files)[1] == '.zip' or os.path.splitext(files)[1] == '.tar':
                    software = software + '/' + files
        else:
            return False

    xml_path = re.sub(r'zip|tar', 'xml', software)
    # xml_path = sw_path.split('/')[-1].split('.')[0] + '.xml'
    if os.path.exists(software) and os.path.exists(xml_path):
        sw = pyocs_software.PyocsSoftware()
        current_test_type = sw.get_test_type_from_ocs(ocs)
        if current_test_type is None:
            current_test_type = "当前OCS为样机or无软件"
        while 1:
            click.echo("测试类型如下：\n"
                       "A : A类 - 全面测试\n"
                       "B : B类 - 针对遥控器的测试\n"
                       "C : C类 - 局部修改针对性测试\n"
                       "D : D类 - 测配屏\n"
                       "E : E类 - 首测，无任何功能及BUG的修改\n"
                       "F : F类 - 过硬件首测，只要求匹配硬件\n"
                       "G : G类 - 仅测试BOOT\n"
                       "BC : B+C - 新遥控器测试，局部功能修改/添加\n"
                       "N : 不用测试\n"
                       "OCS最新软件测试类型：" + current_test_type + " \n ")
            test_type = input("请输入测试类型(N,A,B,C,D,E,F,G,BC)：")
            test_type = test_type.upper()
            if test_type not in {'N', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'BC'}:
                click.echo("测试类型输入错误，请重新输入")
                continue
            message = input("请输入软件备注：")
            ret = sw.upload_software_to_ocs(ocs_num=ocs, zip_path=software,
                                            xml_path=xml_path, test_type=sw.sw_test_type[test_type], message=message)
            sw_name = software.split('/')[-1]
            if ret:
                while sw_name\
                        not in sw.get_enable_software_list_from_html(PyocsDemand(ocs_number=ocs).get_ocs_html()):
                    continue
                else:
                    sw_dlink = sw.get_software_download_link(ocs_number=ocs, sw_name=sw_name)
                    click.echo("--------------------XML不匹配项----------------------")
                    xml_echo_flg = True
                    audit_link = sw.get_enable_sw_audit_link(ocs_number=ocs)
                    if audit_link is None:
                        xml_echo_flg = False
                    else:
                        audit_msg = sw.get_self_audit_msg(audit_link_str=audit_link)
                        if audit_msg is None:
                            xml_echo_flg = False
                    if xml_echo_flg:
                        for msg in audit_msg:
                            click.echo(msg)
                    click.echo("-----------------------------------------------------")
                    click.echo(use_style("软件包名：" + sw_dlink.name, mode="bold"))
                    click.echo(use_style("下载链接：" + sw_dlink.download_link +
                                         " 有效截止时间：" + sw_dlink.deadline, mode="bold"))
                    click.echo("-----------------------------------------------------")
                    click.echo("上传完成")
            else:
                click.echo("上传失败")
            break
    else:
        click.echo("本地未找到上传软件")
        click.echo("确认软件包路径是否正确：" + software)
        click.echo("确认xml路径是否正确：" + xml_path)


@click.command()
@click.option('--src', help='源订单号')
@click.option('--dst', help='目标订单号')
@click.option('--workspace', help='工作区（jenkins参数）')
def copy(src, dst, workspace):
    """从源订单上拷贝所有启用状态下的库存软件到目标订单上"""
    try:
        dst_ocs_list = dst.split(";")
        if not workspace:
            workspace = os.getcwd()
        for dst_ocs in dst_ocs_list:
            sw = pyocs_software.PyocsSoftware()
            sw.reuse_old_sw_from_src_to_dst(src_ocs=src, dst_ocs=dst_ocs, workspace=workspace)
    except NoSoftwareError:
        click.echo("Fail:源订单上无软件")
    except ReviewExcelError:
        click.echo("Fail:客户批号信息不匹配，请检查并重新上传Excel")


@click.command()
@click.option('--ocs', help='订单号')
@click.argument("color")
def mark(ocs, color):
    """标记订单颜色"""
    de = pyocs_software.PyocsDemand(ocs_number=ocs)
    de.set_flag_color(color)


cli.add_command(require)
cli.add_command(jenkins)
cli.add_command(download)
cli.add_command(refresh)
cli.add_command(autobin)
cli.add_command(resource)
cli.add_command(autoconf)
cli.add_command(update)
cli.add_command(comment)
cli.add_command(upload)
cli.add_command(autodt)
cli.add_command(copy)
cli.add_command(mark)

if __name__ == '__main__':
    cli()
