from projectRule.msd358.Msd358Common import Msd358Common
from pyocs import pyocs_confluence
import re

class Ruler(Msd358Common):

    # Customer_ID
    _customer_id = 'CUSTOMER_JINPIN'

    # 代码分支
    _code_branch = ''

    # 测试类型
    _test_type = 'E'

    # 安卓系统
    _android_system = ''

    def get_ocs_modelid(self):
        project = self.request_dict[self.ocs_demand.product_name].replace('.', '_')
        region_name_list = self.request_dict[self.ocs_demand.region_name]
        kb = pyocs_confluence.PyocsConfluence()
        country = kb.get_country_name_by_country(region_name_list)
        if not country:
            country = 'DUBAI'
        batch_code = self.request_dict[self.ocs_demand.customer_batch_code]
        batch_code = re.sub("\D|'-'", "", batch_code)
        batch_code = batch_code.replace('-', '_')
        machine = self.request_dict[self.ocs_demand.customer_machine]
        machine = re.sub(u"([^\u4e00-\u9fa5\u0030-\u0039\u0041-\u005a\u0061-\u007a])", '', machine)
        machine = machine.replace('.', '_')
        modelid = 'CP' + self.ocs_number + '_JPE_' + project + '_' + country + '_PT430CT01_1' + '_JPE_' + batch_code + '_' + machine
        return modelid

    def get_ocs_require(self):
        """获取ocs上的配置，生成配置代码
        Args:
            ocs_number：OCS订单号
        Returns:
             返回配置代码
        """
        ret = ''
        _space = 60
        ret += '#elif ( IsModelID('+ self.get_ocs_modelid() + ') )' + '\n'
        ret += '// hardware & toll item' + '\n'
        ret += self.get_board_macro()
        ret += self.get_chip_macro()
        ret += self.get_tv_system_macro()
        ret += self.get_flash_size_macro()
        ret += self.get_pwm_macro()
        ret += self.get_eshare_macro()

        # 金品代测订单不需要关注IR/KEYPAD/LOGO等的配置项
        ret += '//ir & keypad & logo' + '\n'
        ret += '#define CVT_DEF_IR_TYPE'.ljust(_space) + 'ID_IR_JP_IPTV_AP_RS41_81_41628W_0033' + '\n'
        ret += '#define CVT_DEF_LOGO_TYPE'.ljust(_space) + 'ID_LOGO_TYPE_JINPIN_JPE_JOY_1366' + '\n'

        # 金品代测订单不需要关注PANEL
        ret += '// panel & pq' + '\n'
        ret += '#define CVT_DEF_STR_PANEL_NAME'.ljust(_space) + '"PT430CT01_1"' + '\n'
        ret += '#define CVT_DEF_PANEL_TYPE'.ljust(_space) + 'ID_PNL_General_1920_1080' + '\n'
        ret += '#define CVT_DEF_PQ_TYPE'.ljust(_space) + 'ID_PQ_JINPIN_PT430CT01_1' + '\n'

        ret += '// language & country & region' + '\n'
        if self._android_system == self.android_4_4 and self._code_branch == 'fae':
            ret += '#define CVT_EN_LANG_ENGLISH_US'.ljust(_space) + '1' + '\n'
            ret += '#define CVT_EN_LANG_CHINESE_CN'.ljust(_space) + '1' + '\n'
            ret += '#define CVT_DEF_LANGUAGE_SELECT'.ljust(_space) + 'ID_LANGUAGE_CHINESE_CN' + '\n'
            ret += '#define CVT_DEF_COUNTRY_SELECT'.ljust(_space) + 'ID_COUNTRY_CHINA' + '\n'
        else:
            ret += '#define CVT_EN_LANG_ENGLISH_US'.ljust(_space) + '1' + '\n'
            ret += '#define CVT_DEF_LANGUAGE_SELECT'.ljust(_space) + 'ID_LANGUAGE_ENGLISH_US' + '\n'
            region_name_list = self.request_dict[self.ocs_demand.region_name]
            kb = pyocs_confluence.PyocsConfluence()
            country = kb.get_country_name_by_country(region_name_list)
            if not country:
                country = 'DUBAI'
            if 'DUBAI' == country:
                ret += '#define CVT_DEF_COUNTRY_SELECT'.ljust(self._space) + 'ID_COUNTRY_ARAB' + '\n'
            else:
                ret += '#define CVT_DEF_COUNTRY_SELECT'.ljust(self._space) + 'ID_COUNTRY_' + country + '\n'

        ret += '// launcher & menu config' + '\n'
        if self._android_system == self.android_8_0:  # 只有8.0导入了GAIA
            # GAIA
            other_app_str = self.request_dict[self.ocs_demand.other_app_soft]
            if other_app_str and other_app_str.rfind('Gaia 2.0', 0, len(other_app_str) - 1) != -1:
                ret += '#define CVT_DEF_LAUNCHER_SELECT'.ljust(self._space) + 'ID_LAUNCHER_30_GAIA_20_JINPIN' + '\n'
                ret += '#define CVT_DEF_SKIN_TYPE'.ljust(self._space) + 'SKIN_JINPIN_INTER_COMMON' + '\n'
            elif other_app_str and other_app_str.rfind('Gaia AI', 0, len(other_app_str) - 1) != -1:
                ret += '#define CVT_EN_APP_CVT_TV_SPEECH'.ljust(self._space) + '1' + '\n'
                ret += '#define CVT_DEF_LAUNCHER_SELECT'.ljust(self._space) + 'ID_LAUNCHER_30_GAIA_20_JINPIN' + '\n'
                ret += '#define CVT_DEF_SKIN_TYPE'.ljust(self._space) + 'SKIN_JINPIN_INTER_COMMON' + '\n'
            elif other_app_str and other_app_str.rfind('Gaia 1.0', 0, len(other_app_str) - 1) != -1:
                ret += '#define CVT_DEF_LAUNCHER_SELECT'.ljust(self._space) + 'ID_LAUNCHER_30_GAIA_10_OVERSEA_FALL' + '\n'
            else:
                ret += '#define CVT_DEF_LAUNCHER_SELECT'.ljust(_space) + 'ID_LAUNCHER_30_NON_GAIA_SPACE' + '\n'
        elif self._android_system == self.android_4_4 and self._code_branch == 'fae':
            ret += '#define CVT_DEF_LAUNCHER_SELECT'.ljust(_space) + 'ID_LAUNCHER_CVTE_LAUNCHER3_CHINA_QIYI' + '\n'
        else:
            pass

        ret += '// special request' + '\n'
        ret += '//end\n'
        return ret

    def get_code_branch(self):
        if self.ocs_demand.get_region_name() == "中国":
            self._code_branch = 'fae'
        elif self.ocs_demand.get_ddr_size() == '512MB':
            self._code_branch = 'oversea'
        elif self.ocs_demand.get_flash_size() == '4GByte':
            self._code_branch = 'oversea'
        elif self.ocs_demand.get_flash_size() == '8GByte':
            self._code_branch = 'fae'
        else:
            raise RuntimeError('代码选择不在规则之内')
        return self._code_branch

    def get_android_system(self):
        if self.ocs_demand.get_region_name() == "中国":
            self._android_system = self.android_4_4
        elif self.ocs_demand.get_ddr_size() == '512MB':
            self._android_system = self.android_4_4
        elif self.ocs_demand.get_flash_size() == '4GByte':
            self._android_system = self.android_4_4
        elif self.ocs_demand.get_flash_size() == '8GByte':
            self._android_system = self.android_8_0
        else:
            raise RuntimeError('代码选择不在规则之内')
        return self._android_system
