# 依赖包
import http
import http.cookiejar
import os

import requests  # https://github.com/kennethreitz/requests
import pathlib
import json
import base64
import logging
from lxml import etree
import platform
...
# @author:zhubowen3432
# @作用：获取可以用于登陆的cookies
# @className：PyocsLogin
# @对外接口：get_login_cookies() 返回request需要的RequestsCookieJar类型的cookies
...


class PyocsLogin:
    # 全局信息
    __Login_Address = "http://ocs.gz.cvte.cn/users/sso_login"  # OCS登陆站点
    __OCS_Test_URL = "http://ocs.gz.cvte.cn/tv"  # 测试用站点
    Access_URL = 'http://ocs.gz.cvte.cn/tv'  # 任何你想要访问的OCS站点
    _instance = None
    _cookies_file = ""
    _str_Location = 'Location'
    _logger = logging.getLogger(__name__)
    
    def __init__(self):
        self._logger.setLevel(level=logging.WARNING)  # 控制打印级别
        if platform.system() == "Linux":
            home_dir = os.environ['HOME']
            self._cookies_file = home_dir + '/.cookies'
            self._logger.info("linux环境")
        else:
            self._cookies_file = 'cookies'
            self._logger.info("windows环境")

    # 单例模式
    def __new__(cls, *args, **kw):
        if not cls._instance:
            cls._instance = super(PyocsLogin, cls).__new__(cls, *args, **kw)
        return cls._instance

    # 因为域账号统一，所以可以为其他系统提供账号，例如jenkins
    @staticmethod
    def get_account_from_json_file():
        if platform.system() == "Linux":
            home_dir = os.environ['HOME']
            _account_json_file = home_dir + '/.account.json'
        else:
            _account_json_file = 'account.json'
        try:
            logging.info(_account_json_file)
            with open(_account_json_file, 'r') as load_file:
                tmp = load_file.read()
                account_dict = json.loads(json.dumps(eval(tmp)))
                return account_dict
        except FileNotFoundError:
            logging.error("请创建JSON格式的文件，包含Username、Password值")
            return None

    def _get_cookies_form_4a(self, username, password):
        # 直接从4a系统获取登陆cookies，主要分为如下4个部分
        # part 1 : 从登陆界面获取sso登陆信息：sso_html、sso_cookie, sso_lt、sso_execution
        sso_session = requests.get(self.__Login_Address)
        html = etree.HTML(sso_session.text)
        self._logger.info("part 1 status code: " + str(sso_session.status_code))
        sso_url = str(sso_session.url)
        self._logger.info("sso_url: " + sso_url)
        sso_cookie = sso_session.cookies
        self._logger.info(sso_cookie)
        sso_lt_list = html.xpath("//input[@name='lt']/@value")
        sso_lt = str(sso_lt_list[0])
        self._logger.info("sso_lt: " + sso_lt)
        sso_execution_list = html.xpath("//input[@name='execution']/@value")
        sso_execution = str(sso_execution_list[0])
        self._logger.info("sso_execution: " + sso_execution)

        # part 2 : 准备表单数据
        form_data = dict()
        form_data.update({
            'username': 'Username',
            'password': 'Password',
            'imageCode': '',
            'lt': 'LT',
            'execution': 'Execution',
            '_eventId': 'submit'
        })
        form_data['username'] = username
        form_data['password'] = password
        form_data['lt'] = sso_lt
        form_data['execution'] = sso_execution
        self._logger.info(form_data)

        # 提交表单数据，获取ticket url
        sso_login_response = requests.post(sso_url, data=form_data, cookies=sso_cookie, allow_redirects=False)
        self._logger.info("part 2 status code: " + str(sso_login_response.status_code))
        ticket_url = sso_login_response.headers[self._str_Location.title()]
        self._logger.info("ticket_url: " + ticket_url)

        # part 3 : 从ticket url获取登陆cookies与跳转url
        ticket_response = requests.get(ticket_url, allow_redirects=False)
        self._logger.info("part 3 status code: " + str(ticket_response.status_code))
        login_cookies = ticket_response.cookies
        self._logger.info(login_cookies)
        login_url = ticket_response.headers[self._str_Location.title()]
        self._logger.info("login_url: " + login_url)

        # part 4 : 获取最终的登陆cookies
        final_session = requests.Session()
        final_response = final_session.get(login_url, cookies=login_cookies, allow_redirects=False)
        self._logger.info("part 4 status code: " + str(final_response.status_code))
        server_cookies = final_response.cookies
        lwp_cookiesjar = http.cookiejar.LWPCookieJar(self._cookies_file)  # 实例化一个LWPcookiejar对象,用于写cookies到文件
        requests.utils.cookiejar_from_dict({c.name: c.value for c in server_cookies}, lwp_cookiesjar)
        lwp_cookiesjar.save(self._cookies_file, ignore_discard=True, ignore_expires=True)
        self._logger.info(server_cookies)
        return server_cookies

    def _get_cookies_from_file(self):
        load_cookiejar = http.cookiejar.LWPCookieJar()
        try:
            # 从文件中加载cookies(LWP格式)
            load_cookiejar.load(self._cookies_file, ignore_discard=True, ignore_expires=True)
        except (FileNotFoundError, http.cookiejar.LoadError):
            self._logger.info("cookies文件未找到")
            return None
        else:
            # 工具方法转换成字典
            load_cookies = requests.utils.dict_from_cookiejar(load_cookiejar)
            # 工具方法将字典转换成RequestsCookieJar，赋值给session的cookies.
            file_cookies = requests.utils.cookiejar_from_dict(load_cookies)
            self._logger.info(file_cookies)
            return file_cookies

    def _use_4a_to_login(self):
        self._logger.info("从4A系统获取cookies登陆")
        account = self.get_account_from_json_file()
        if account is None:
            return None
        username = account['Username']
        base64_password = base64.b64encode(account['Password'].encode())
        tmp_cookies = self._get_cookies_form_4a(username, base64_password)
        request_from_4a = requests.get(self.Access_URL, cookies=tmp_cookies)
        if str(request_from_4a.url) == self.Access_URL:
            self._logger.info('远程cookies登陆成功')
            return tmp_cookies

    def get_login_cookies(self):
        """
        :rtype: RequestsCookieJar
        """
        cookie_file = pathlib.Path(self._cookies_file)
        if cookie_file.exists():
            file_cookies = self._get_cookies_from_file()
            self._logger.info("本地cookies:" + str(file_cookies))
            r = requests.get(self.Access_URL, cookies=file_cookies)
            self._logger.info("r.url:" + r.url)
            if str(r.url) == self.Access_URL:
                self._logger.info('本地cookies登陆成功')
                return file_cookies
            else:
                self._logger.info('本地cookies过期，从4A重新获取cookies')
                # cookie_file.unlink()
                return self._use_4a_to_login()
        else:
            cookie_file.open(mode='w')  # 创建cookies文件
            return self._use_4a_to_login()
